package jp.co.yahoo.hadoop.common;

import java.io.IOException;

import org.apache.hadoop.io.NullWritable;
import org.apache.hadoop.io.BytesWritable;

import org.apache.hadoop.mapreduce.InputSplit;
import org.apache.hadoop.mapreduce.RecordReader;
import org.apache.hadoop.mapreduce.TaskAttemptContext;

import org.apache.hadoop.fs.FSDataInputStream;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.IOUtils;

/**
 * An RecordReader used with CombileWholeFileInputFormat.
 */
public class CombineWholeFileRecordReader extends
		RecordReader<NullWritable, BytesWritable> {
	private Path path;
	private FileSystem fs;
	private int length;
	private boolean processed = false;

	private NullWritable key = NullWritable.get();
	private BytesWritable value = new BytesWritable();
	
	public CombineWholeFileRecordReader(CombineFileSplit split,
	        TaskAttemptContext context, Integer index) throws IOException {
		fs = FileSystem.get(context.getConfiguration());
		path = split.getPath(index);
		long startOffset = split.getOffset(index);
		if(startOffset != 0)
			processed = true;
		length = (int)split.getLength(index);
	}
	
	@Override
	public void initialize(InputSplit inputSplit,
			TaskAttemptContext taskAttemptContext) throws IOException,
			InterruptedException {
	}
	
	@Override
	public boolean nextKeyValue() throws IOException {
		if (!processed) {
			byte[] contents = new byte[length];
			FSDataInputStream in = null;
			try {
				in = fs.open(path);
				IOUtils.readFully(in, contents, 0, contents.length);
				value.set(contents, 0, contents.length);
			} finally {
				IOUtils.closeStream(in);
			}
			processed = true;
			return true;
		}
		return false;
	}

	@Override
	public NullWritable getCurrentKey() throws IOException,
			InterruptedException {
		return key;
	}

	@Override
	public BytesWritable getCurrentValue() throws IOException,
			InterruptedException {
		return value;
	}

	@Override
	public float getProgress() throws IOException, InterruptedException {
		return processed ? 1.0f : 0.0f;
	}

	@Override
	public void close() throws IOException {
	}
}
