package jp.co.yahoo.hadoop.ml.aozora;

import jp.co.yahoo.hadoop.common.CombineWholeFileInputFormat;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.MapWritable;
import org.apache.hadoop.io.VIntWritable;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
import org.apache.hadoop.mapreduce.lib.output.SequenceFileOutputFormat;
import org.apache.hadoop.util.GenericOptionsParser;

/**
 * A MapReduce driver for extracting the features of books in Aozora-Bunko.
 */
public class ExtractAozoraFeatureDriver {
	public static void main(String[] args) throws Exception {
		Configuration conf = new Configuration();

		String[] otherArgs = new GenericOptionsParser(conf, args)
				.getRemainingArgs();
		if (otherArgs.length != 2) {
			System.err.println("Usage: hadoop <in> <out>");
			System.exit(2);
		}

		Job job = new Job(conf, "extract aozora feature");
		job.setJarByClass(ExtractAozoraFeatureDriver.class);
		job.setMapperClass(ExtractAozoraFeatureMapper.class);
		job.setInputFormatClass(CombineWholeFileInputFormat.class);
		job.setOutputFormatClass(SequenceFileOutputFormat.class);
		job.setOutputKeyClass(VIntWritable.class);
		job.setOutputValueClass(MapWritable.class);
		job.setNumReduceTasks(0);
		FileInputFormat.addInputPath(job, new Path(otherArgs[0]));
		FileOutputFormat.setOutputPath(job, new Path(otherArgs[1]));
		System.exit(job.waitForCompletion(true) ? 0 : 1);
	}
}