package jp.co.yahoo.hadoop.ml.classifier;

import java.io.IOException;
import java.util.Map;
import java.util.TreeMap;

import jp.co.yahoo.hadoop.ml.common.CommonUtil;
import jp.co.yahoo.hadoop.ml.common.DiscriminantFunctionAlgorithm;

import org.apache.hadoop.io.MapWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.VIntWritable;

/**
 * A test class for multi-class classification
 */
public class TestMulticlassClassifier {
	private static int classWidth = 5;
	
	public static void main(String[] args) throws IOException {
		if (args.length < 2) {
			System.err.println("Usage: cmd <test file> <train file>");
			return;
		}

		MapWritable testMap = CommonUtil.readTestFile(args[0]);
		Map<VIntWritable, MapWritable> weightMap = CommonUtil
				.readMulticlassWeightFile(args[1]);

		testMap.put(new Text(CommonUtil.BIAS_KEY), new VIntWritable(1));

		TreeMap<Double, Integer> yMap = new TreeMap<Double, Integer>();

		for (VIntWritable cat : weightMap.keySet()) {
			double yk = Math.exp(DiscriminantFunctionAlgorithm.predict(testMap,
					weightMap.get(cat)));
			yMap.put(yk, cat.get());
		}

		double resultSum = 0;
		double ySum = 0;
		int catCount = 0;
		for (Double k : yMap.descendingKeySet()) {
			if (catCount >= 2)
				break;
			resultSum += (yMap.get(k) * classWidth + 2.5) * k;
			ySum += k;
			System.out.println(catCount + ":"
					+ (yMap.get(k) * classWidth + 2.5));
			catCount++;
		}
		double result = resultSum / ySum;

		System.out.println("あなたは" + result + "歳で死ぬでしょう。");
	}
}
