package jp.co.yahoo.hadoop.ml.common;

import java.util.Map.Entry;

import org.apache.hadoop.io.DoubleWritable;
import org.apache.hadoop.io.MapWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.VIntWritable;
import org.apache.hadoop.io.Writable;

/**
 * A utility class for discriminant function algorithm
 */
public class DiscriminantFunctionAlgorithm {

	private DiscriminantFunctionAlgorithm() {
		throw new AssertionError();
	}

	/**
	 * predict a value with liner discriminant function.
	 * 
	 * @param train
	 *            a feature vector of training data
	 * @param weight
	 *            a weight vector
	 * @return the predicted value
	 */
	public static double predict(MapWritable train, MapWritable weight) {
		double result = 0;
		for (Entry<Writable, Writable> entry : train.entrySet()) {
			Text word = (Text) entry.getKey();
			int x = ((VIntWritable) entry.getValue()).get();
			DoubleWritable w = (DoubleWritable) weight.get(word);
			if (w != null)
				result += x * w.get();
		}
		return result;
	}
	
	/**
	 * predict a value with logistic discriminant function.
	 * 
	 * @param train
	 *            a feature vector of training data
	 * @param weight
	 *            a weight vector
	 * @return the predicted value
	 */
	public static double predictLogistic(MapWritable train, MapWritable weight,
			double multiplySize) {
		double result = predict(train, weight);
		if (result < -100) {
			result = 0;
		} else {
			result = multiplySize / (1.0 + Math.exp(-result));
		}
		return result;
	}
}