package jp.co.yahoo.hadoop.ml.common;

import java.io.IOException;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.util.GenericOptionsParser;

/**
 * An abstract class for MapReduce driver using Iterative Parameter Mixing
 */
public abstract class IterativeParameterMixingDriver {

	/**
	 * Run MapReduce job
	 * 
	 * @param args
	 *            command line arguments
	 * @throws Exception
	 */
	public void run(String[] args) throws Exception {
		Configuration conf = new Configuration();

		String[] otherArgs = new GenericOptionsParser(conf, args)
				.getRemainingArgs();
		if (otherArgs.length < 2) {
			System.err
					.println("Usage: hadoop <input file> <output dir> [algorithm]");
			System.exit(2);
		}

		String algorithm = "default";
		if (otherArgs.length >= 3) {
			algorithm = otherArgs[2];
		}

		int maxIteration = conf.getInt("iteration.num", 30);

		Path input = new Path(otherArgs[0]);
		int iteration = 1;
		while (maxIteration >= iteration) {
			Path output = new Path(otherArgs[1], "weight-" + iteration);
			if (iteration > 1) {
				Path prevWeight = new Path(otherArgs[1], "weight-"
						+ (iteration - 1) + "/part-r-00000");
				conf.set("weight.file", prevWeight.toString());
			}
			runIteration(conf, input, output, algorithm);
			iteration++;
		}
	}

	protected abstract void runIteration(Configuration conf, Path input,
			Path output, String algorithm) throws IOException,
			InterruptedException, ClassNotFoundException;
}