package jp.co.yahoo.hadoop.ml.regression;

import java.io.IOException;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.DoubleWritable;
import org.apache.hadoop.io.MapWritable;
import org.apache.hadoop.io.NullWritable;
import org.apache.hadoop.io.SequenceFile;
import org.apache.hadoop.io.VIntWritable;
import org.apache.hadoop.mapreduce.Mapper;

/**
 * A base mapper class for regression
 */
public abstract class RegressionMapper extends
		Mapper<VIntWritable, MapWritable, DoubleWritable, MapWritable> {
	protected MapWritable weightMap = new MapWritable();
	protected int trainSize = 0;
	protected double diffSum = 0;

	@Override
	public void setup(Context context) throws IOException, InterruptedException {
		Configuration conf = context.getConfiguration();
		String weightFile = conf.get("weight.file");
		if (weightFile != null) {
			FileSystem fs = FileSystem.get(conf);
			SequenceFile.Reader reader = new SequenceFile.Reader(fs, new Path(
					weightFile), conf);
			try {
				NullWritable key = NullWritable.get();
				reader.next(key, weightMap);
			} finally {
				reader.close();
			}
		}
	}
	
	@Override
	public void cleanup(Context context) throws IOException,
			InterruptedException {
		double mae = diffSum / trainSize;
		System.out.println("MAE: " + mae);
		context.write(new DoubleWritable(mae), weightMap);
	}
}